@@ -15,7 +15,7 @@ Command details:
 Usage:
     picpro program -p PORT -i HEX_FILE -t PIC_TYPE [--id=PIC_ID] [--fuse=FUSE_NAME:FUSE_VALUE...] [--icsp]
     picpro verify -p PORT -i HEX_FILE -t PIC_TYPE [--icsp]
-    picpro dump <mem_type> -p PORT -b BIN_FILE -t PIC_TYPE [--icsp]
+    picpro dump <mem_type> -p PORT -o HEX_FILE -t PIC_TYPE [--icsp]
     picpro chipinfo [<PIC_TYPE>]
     picpro (-h | --help)
 
@@ -27,7 +27,7 @@ Options:
     -p PORT --port=PORT              Set serial port where programmer is connected.
     -t PIC_TYPE --pic_type=PIC_TYPE  Pic type you are programming/reading.
     -i HEX_FILE --hex_file=HEX_FILE  Hex file to flash or to read.
-    -b BIN_FILE --bin_file=BIN_FILE  Bin file path.
+    -o HEX_FILE --hex_file=HEX_FILE  Hex file to write.
 """
 
 import os.path
@@ -38,13 +38,14 @@ import json
 from functools import wraps
 from typing import Union, Optional, Callable, Tuple
 import serial
+from intelhex import IntelHex
 from docopt import docopt
 from picpro.ChipInfoReader import ChipInfoReader
 from picpro.IChipInfoEntry import IChipInfoEntry
 from picpro.HexFileReader import HexFileReader
 from picpro.ProtocolInterface import ProtocolInterface
 from picpro.exceptions import FuseError, InvalidResponseError
-from picpro.tools import range_filter_records, swab_record, merge_records
+from picpro.tools import range_filter_records, swab_record, merge_records, swab_bytes
 import picpro as app_root
 
 APP_ROOT_FOLDER = os.path.abspath(os.path.dirname(app_root.__file__))
@@ -134,18 +135,27 @@ def dump() -> None:
     _, protocol_interface = programmer_common_data
 
     mem_type = OPTIONS['<mem_type>']
-    output_file = OPTIONS['--bin_file']
+    output_file = OPTIONS['--hex_file']
 
-    with open(output_file, 'wb') as file:
+    if mem_type == 'eeprom':
+        print('Reading EEPROM into file {}...'.format(output_file))
+        content = swab_bytes(protocol_interface.read_eeprom())
+    elif mem_type == 'rom':
+        print('Reading ROM into file {}...'.format(output_file))
+        content = swab_bytes(protocol_interface.read_rom())
+    elif mem_type == 'config':
+        print('Reading CONFIG into file {}...'.format(output_file))
+        content = protocol_interface.read_config()
+        print(content)
+        return
+    else:
+        raise ValueError('Unknown memory type')
 
-        if mem_type == 'eeprom':
-            print('Reading EEPROM into file {}...'.format(output_file))
-            file.write(protocol_interface.read_eeprom())
-        elif mem_type == 'rom':
-            print('Reading ROM into file {}...'.format(output_file))
-            file.write(protocol_interface.read_rom())
-        else:
-            raise ValueError('Unknown memory type')
+    intel_hex = IntelHex()
+    intel_hex.frombytes(content)
+
+    with open(output_file, 'w') as file:
+        intel_hex.write_hex_file(file)
 
 
 @command()
@@ -286,9 +296,10 @@ def prepare_flash_data_from_hex_file(chip_info: IChipInfoEntry, hex_file: HexFil
                     swap_bytes = True
                     break
                 if not (le_ok or be_ok):
-                    raise ValueError('Invalid ROM word: {}, ROMblank: {}'.format(hex(le_word), hex(rom_blank_word)))
+                    raise ValueError('Invalid ROM word: {}, ROM blank: {}'.format(hex(le_word), hex(rom_blank_word)))
         if swap_bytes is not None:
             break
+
     if swap_bytes:
         rom_records = [*map(swab_record, rom_records)]
         config_records = [*map(swab_record, config_records)]
@@ -312,32 +323,32 @@ def prepare_flash_data_from_hex_file(chip_info: IChipInfoEntry, hex_file: HexFil
 
     # FINALLY!  We create the byte-level data...
     rom_data = merge_records(rom_records, rom_blank)
-
     eeprom_data = merge_records(eeprom_records, eeprom_blank, 0x4200)
 
     # Extract fuse data, pic_id data, etc. from fuse records
     if pic_id is not None:
         id_data = pic_id.encode('UTF-8')
     else:
-        id_data = merge_records(
-            range_filter_records(config_records, 0x4000, 0x4008),
-            (b'\x00' * 8)
-        )
+        id_data_raw = range_filter_records(config_records, 0x4000, 0x4008)
+        id_data = merge_records(id_data_raw, (b'\x00' * 8), 0x4000)
         # If this is a 16-bit core, leave id_data as-is.  Else we need to
         # take every other byte according to endian-ness.
-        if chip_info.get_core_bits() != 16:
-            id_data = bytearray([id_data[x] for x in range(pick_byte, 8, 2)])
+        # Config records are already converted to little-endian, so we set
+        # range(pick_byte, 8, 2) to range(1, 8, 2)
+        if core_bits != 16:
+            id_data = bytearray([id_data[x] for x in range(1, 8, 2)])
 
     # Pull fuse data from config records
-    fuse_data_list = list(map(lambda word: struct.pack('>H', word), chip_info.programming_vars.fuse_blank))
-    fuse_data = b''.join(fuse_data_list)
+    fuse_data_blank = b''.join(map(lambda word: struct.pack('>H', word), chip_info.programming_vars.fuse_blank))
+    fuse_config = range_filter_records(
+        config_records,
+        0x400e,
+        0x4010
+    )
+
     fuse_data = merge_records(
-        range_filter_records(
-            config_records,
-            0x400e,
-            0x4010
-        ),
-        fuse_data,
+        fuse_config,
+        fuse_data_blank,
         0x400e
     )
 
@@ -390,9 +401,9 @@ def program_pic(
 
     try:
         # If write mode is active, program the ROM, EEPROM, ID and fuses.
+        chip_config = protocol_interface.read_config()
         if is_program:
             # Write ROM, EEPROM, ID and fuses
-
             print('Erasing Chip')
             if not protocol_interface.erase_chip():
                 print('Erasure failed.')
@@ -416,15 +427,27 @@ def program_pic(
                 print('Programming ID and fuses failed.')
                 return False
 
+            print('Programming CAL and fuses')
+            response = protocol_interface.program_calibration(chip_config.get('calibrate'), fuse_values[0])
+            if response == 'C':
+                print('Programming CAL failed.')
+                return False
+
         # Verify programmed data.
         # Behold, my godlike powers of verification:
         print('Verifying ROM')
         pic_rom_data = protocol_interface.read_rom()
         verification_result = True
 
+        if chip_info.programming_vars.flag_calibration_value_in_rom:
+            # Some chips have cal data put on last two bytes of ROM dump
+            rom_data = rom_data[:len(rom_data) - 2] + chip_config.get('calibrate').to_bytes(2, 'big')
+
         if pic_rom_data == rom_data:
             print('ROM verified.')
         else:
+            print('{} {}'.format(pic_rom_data[-1], pic_rom_data[-2]))
+            print('{} {}'.format(rom_data[-1], rom_data[-2]))
             print('ROM verification failed.')
             verification_result = False
 
